import * as d3 from "d3";
const COLORS = {
    links: "#C8C8C8",
    nodeHover: "#A9A9A9",
    node: (d) => {
        return d.isMain ? "hsl(210, 70%, 72.5%)" : "hsl(55, 70%, 72.5%)";
    },
};
function handleNodeMouseOver() {
    d3.select(this).style("fill", COLORS.nodeHover);
}
function handleNodeMouseOut(_event, d) {
    d3.select(this).style("fill", COLORS.node(d));
}
function getLinkText(d) {
    return `${d.source.id}-${d.target.id}`;
}
function getNodeId(d) {
    return d.id;
}
function removeD3Item() {
    d3.select(this).remove();
}
export const updateGraph = (newNodes, newLinks, simulation, svgElement) => {
    if (!simulation || !svgElement)
        return;
    simulation.nodes(newNodes);
    simulation.force("link")?.links(newLinks);
    simulation.alpha(0.5).restart();
    const link = d3
        .select(svgElement)
        .select(".links")
        .selectAll("line")
        .data(newLinks, getLinkText);
    link
        .enter()
        .append("line")
        .merge(link)
        .attr("stroke", COLORS.links)
        .transition()
        .duration(500)
        .attr("stroke-width", 0.5);
    link
        .exit()
        .transition()
        .duration(200)
        .style("opacity", 0)
        .on("end", removeD3Item);
    const node = d3
        .select(svgElement)
        .select(".nodes")
        .selectAll("circle")
        .data(newNodes, getNodeId);
    node
        .enter()
        .append("circle")
        .merge(node)
        .attr("r", (d) => (d.isMain ? 15 : 13))
        .attr("fill", (d) => COLORS.node(d))
        .on("mouseover", handleNodeMouseOver)
        .on("mouseout", handleNodeMouseOut)
        .call(drag(simulation));
    node.exit().transition().duration(200).attr("r", 0).remove();
    const text = d3
        .select(svgElement)
        .select(".nodes")
        .selectAll("text")
        .data(newNodes, getNodeId);
    text
        .enter()
        .append("text")
        .style("fill-opacity", 0)
        .merge(text)
        .text(getNodeId)
        .style("text-anchor", "middle")
        .style("font-size", "12px")
        .style("font-family", "sans-serif")
        .transition()
        .duration(500)
        .style("fill-opacity", 1);
    text
        .exit()
        .transition()
        .duration(200)
        .style("fill-opacity", 0)
        .on("end", removeD3Item);
    simulation.on("tick", () => {
        d3.select(svgElement)
            .select(".links")
            .selectAll("line")
            .attr("x1", (d) => d.source.x ?? 0)
            .attr("y1", (d) => d.source.y ?? 0)
            .attr("x2", (d) => d.target.x ?? 0)
            .attr("y2", (d) => d.target.y ?? 0);
        d3.select(svgElement)
            .select(".nodes")
            .selectAll("circle")
            .attr("cx", (d) => d.x ?? 0)
            .attr("cy", (d) => d.y ?? 0);
        d3.select(svgElement)
            .select(".nodes")
            .selectAll("text")
            .attr("x", (d) => d.x ?? 0)
            .attr("y", (d) => (d.y === undefined ? 0 : d.y - 20));
    });
};
const drag = (simulation) => {
    const dragStarted = (event, d) => {
        if (!event.active)
            simulation.alphaTarget(0.3).restart();
        d.fx = d.x;
        d.fy = d.y;
    };
    const dragged = (event, d) => {
        d.fx = event.x;
        d.fy = event.y;
    };
    const dragEnded = (event, d) => {
        if (!event.active)
            simulation.alphaTarget(0);
        d.fx = null;
        d.fy = null;
    };
    return d3
        .drag()
        .on("start", dragStarted)
        .on("drag", dragged)
        .on("end", dragEnded);
};
export const prepareGroups = (svg) => {
    if (d3.select(svg).select("g.links").empty()) {
        d3.select(svg).append("g").attr("class", "links");
    }
    if (d3.select(svg).select("g.nodes").empty()) {
        d3.select(svg).append("g").attr("class", "nodes");
    }
};
export const createSimulation = (width, height) => {
    return d3
        .forceSimulation()
        .force("link", d3.forceLink().id(getNodeId).distance(110))
        .force("charge", d3.forceManyBody())
        .force("center", d3.forceCenter(width / 2, height / 2))
        .force("collide", d3
        .forceCollide()
        .radius((d) => (d.isMain ? 20 : 15))
        .iterations(2));
};
//# sourceMappingURL=network.js.map