import { IndexedDbWrapper } from "./indexed-db-wrapper";
function getStorageItemId(streamId, segmentId) {
    return `${streamId}|${segmentId}`;
}
const INFO_ITEMS_STORE_NAME = "segmentInfo";
const DATA_ITEMS_STORE_NAME = "segmentData";
const DB_NAME = "p2p-media-loader";
const DB_VERSION = 1;
const BYTES_PER_MB = 1048576;
export class IndexedDbStorage {
    constructor() {
        Object.defineProperty(this, "segmentsMemoryStorageLimit", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 4096
        }); // memory storage limit in MiB
        Object.defineProperty(this, "currentMemoryStorageSize", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        }); // current memory storage size in MiB
        Object.defineProperty(this, "storageConfig", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "mainStreamConfig", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "secondaryStreamConfig", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "cache", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        Object.defineProperty(this, "currentPlayback", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        }); // current playback position and rate
        Object.defineProperty(this, "lastRequestedSegment", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        }); // details  about the last requested segment by the player
        Object.defineProperty(this, "db", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "segmentChangeCallback", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.db = new IndexedDbWrapper(DB_NAME, DB_VERSION, INFO_ITEMS_STORE_NAME, DATA_ITEMS_STORE_NAME);
    }
    onPlaybackUpdated(position, rate) {
        this.currentPlayback = { position, rate };
    }
    onSegmentRequested(swarmId, streamId, segmentId, startTime, endTime, streamType, isLiveStream) {
        this.lastRequestedSegment = {
            streamId,
            segmentId,
            startTime,
            endTime,
            swarmId,
            streamType,
            isLiveStream,
        };
    }
    async initialize(storageConfig, mainStreamConfig, secondaryStreamConfig) {
        this.storageConfig = storageConfig;
        this.mainStreamConfig = mainStreamConfig;
        this.secondaryStreamConfig = secondaryStreamConfig;
        try {
            // await this.db.deleteDatabase();
            await this.db.openDatabase();
            await this.loadCacheMap();
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.error("Failed to initialize custom segment storage:", error);
            throw error;
        }
    }
    async storeSegment(swarmId, streamId, segmentId, data, startTime, endTime, streamType, _isLiveStream) {
        const storageId = getStorageItemId(streamId, segmentId);
        const segmentDataItem = {
            storageId,
            data,
        };
        const segmentInfoItem = {
            storageId,
            dataLength: data.byteLength,
            streamId,
            segmentId,
            streamType,
            startTime,
            endTime,
            swarmId,
        };
        try {
            /*
             * await this.clear();
             * Implement your own logic to remove old segments and manage the memory storage size
             */
            await Promise.all([
                this.db.put(DATA_ITEMS_STORE_NAME, segmentDataItem),
                this.db.put(INFO_ITEMS_STORE_NAME, segmentInfoItem),
            ]);
            this.cache.set(storageId, segmentInfoItem);
            this.increaseMemoryStorageSize(data.byteLength);
            if (this.segmentChangeCallback) {
                this.segmentChangeCallback(streamId);
            }
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.error(`Failed to store segment ${segmentId}:`, error);
            throw error;
        }
    }
    async getSegmentData(_swarmId, streamId, segmentId) {
        const segmentStorageId = getStorageItemId(streamId, segmentId);
        try {
            const result = await this.db.get(DATA_ITEMS_STORE_NAME, segmentStorageId);
            return result?.data;
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.error(`Error retrieving segment data for ${segmentStorageId}:`, error);
            return undefined;
        }
    }
    getUsage() {
        /*
         * Implement your own logic to calculate the memory used by the segments stored in memory.
         */
        return {
            totalCapacity: this.segmentsMemoryStorageLimit,
            usedCapacity: this.currentMemoryStorageSize,
        };
    }
    hasSegment(_swarmId, streamId, segmentId) {
        const storageId = getStorageItemId(streamId, segmentId);
        return this.cache.has(storageId);
    }
    getStoredSegmentIds(_swarmId, streamId) {
        const storedSegments = [];
        for (const segment of this.cache.values()) {
            if (segment.streamId === streamId) {
                storedSegments.push(segment.segmentId);
            }
        }
        return storedSegments;
    }
    destroy() {
        this.db.closeDatabase();
        this.cache.clear();
    }
    setSegmentChangeCallback(callback) {
        this.segmentChangeCallback = callback;
    }
    async loadCacheMap() {
        const result = await this.db.getAll(INFO_ITEMS_STORE_NAME);
        result.forEach((item) => {
            const storageId = getStorageItemId(item.streamId, item.segmentId);
            this.cache.set(storageId, item);
            this.increaseMemoryStorageSize(item.dataLength);
        });
    }
    increaseMemoryStorageSize(dataLength) {
        this.currentMemoryStorageSize += dataLength / BYTES_PER_MB;
    }
}
//# sourceMappingURL=indexed-db-storage.js.map