export class IndexedDbWrapper {
    constructor(dbName, dbVersion, infoItemsStoreName, dataItemsStoreName) {
        Object.defineProperty(this, "dbName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: dbName
        });
        Object.defineProperty(this, "dbVersion", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: dbVersion
        });
        Object.defineProperty(this, "infoItemsStoreName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: infoItemsStoreName
        });
        Object.defineProperty(this, "dataItemsStoreName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: dataItemsStoreName
        });
        Object.defineProperty(this, "db", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
    }
    async openDatabase() {
        return new Promise((resolve, reject) => {
            const request = indexedDB.open(this.dbName, this.dbVersion);
            request.onerror = () => reject(new Error("Failed to open database."));
            request.onsuccess = () => {
                this.db = request.result;
                resolve();
            };
            request.onupgradeneeded = (event) => {
                this.db = event.target.result;
                this.createObjectStores(this.db);
            };
        });
    }
    createObjectStores(db) {
        if (!db.objectStoreNames.contains(this.dataItemsStoreName)) {
            db.createObjectStore(this.dataItemsStoreName, { keyPath: "storageId" });
        }
        if (!db.objectStoreNames.contains(this.infoItemsStoreName)) {
            db.createObjectStore(this.infoItemsStoreName, { keyPath: "storageId" });
        }
    }
    async getAll(storeName) {
        return this.performTransaction(storeName, "readonly", (store) => store.getAll());
    }
    async put(storeName, item) {
        await this.performTransaction(storeName, "readwrite", (store) => store.put(item));
    }
    async get(storeName, key) {
        return this.performTransaction(storeName, "readonly", (store) => store.get(key));
    }
    async delete(storeName, key) {
        await this.performTransaction(storeName, "readwrite", (store) => store.delete(key));
    }
    async performTransaction(storeName, mode, operation) {
        return new Promise((resolve, reject) => {
            if (!this.db)
                throw new Error("Database not initialized");
            const transaction = this.db.transaction(storeName, mode);
            const store = transaction.objectStore(storeName);
            const request = operation(store);
            request.onerror = () => reject(new Error("IndexedDB operation failed"));
            request.onsuccess = () => {
                const result = request.result;
                resolve(result);
            };
        });
    }
    closeDatabase() {
        if (!this.db)
            return;
        this.db.close();
        this.db = null;
    }
    async deleteDatabase() {
        this.closeDatabase();
        return new Promise((resolve, reject) => {
            const request = indexedDB.deleteDatabase(this.dbName);
            request.onsuccess = () => resolve();
            request.onerror = () => reject(new Error("Failed to delete database."));
        });
    }
}
//# sourceMappingURL=indexed-db-wrapper.js.map